/*
  Linked List header file
  Documentation is in HeaderDoc format,
  see http://developer.apple.com/darwin/projects/headerdoc/ for more info
 */

/*!
  @header Linked-List
  @abstract Simple Linked-List implementation
  @discussion Each item is a void*, giving the flexability to use any data time. When the list is destroyed, each item is free()d.

  (C)2001-2003 Stewart Smith

  Distributed under the GNU Public License

  See the included LICENSE file for details

 */

#ifndef __LIST_H__
#define __LIST_H__

/*!
  @struct ListNodeStruc
  @abstract Structure for a List Node
  @field item Pointer to List item. Should be malloc()ed externally to the list, as it is free()ed when list_destroy is called.
  @field next pointer to the next ListNode, or NULL if last node.
*/
struct ListNodeStruc{
	void *item;
	struct ListNodeStruc *next;
};

/*!
  @typedef ListNode
  @abstract Defines ListNode as the struct ListNodeStruc
*/
typedef struct ListNodeStruc ListNode;

/*!
  @typedef List
  @abstract Linked List
  @discussion Type for a linked list. Before use, list_new(*List) must be called to initialise the structure.
  @field list Pointer to first ListNode in the list
  @field current pointer to current node in traversal
  @field freeItem Pointer to function (void freeItem(void*)) used to free a node, otherwise NULL.
  @field numItems number of items in the list.
 */
typedef struct {
  ListNode* list;
  ListNode* current;
  void (*freeItem)(void*);
  int numItems; 
} List;

/*!
  @function list_new
  @result integer, true on success
  @param *theList pointer to list to be 'newed'
*/
int list_new(List *theList);

/*!
  @function list_destroy
  @abstract Removes all items from a list.
  @discussion Will traverse all nodes in a list, first free()ing the item, and then removing the node.
 */
void list_destroy(List *theList);

/*!
  @function list_append
  @result true on success
  @param *theList pointer to the list to add the item to
  @param *theItem pointer to void, the item to append.
 */
int list_append(List *theList, void* theItem);

/*!
  @function list_getItem
  @abstract Get an item from the list
  @result pointer to item
  @param theList pointer to list to search
  @param item item to look for (void*)
  @param compare pointer to function to compare item to each node's item. They can be different types, but the compare function must be able to track this itself.
 */
void *list_getItem(List *theList,void* item, int (*compare)(void*,void*));

/*!
  @function list_getID
  @result pointer to item
  @param theList pointer to list to search
  @param id number of items to traverse into the list (from start)
 */
void *list_getID(List *theList,long id);

/*!
  @function list_traverse
  @result next item in list
  @param theList list to traverse
  @param current Status variable (initially should be NULL)
*/
void* list_traverse(List *theList, ListNode** current);

#endif








